package com.malk.shanghaishiwei.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilDateTime;
import com.malk.Util.UtilFile;
import com.malk.Util.UtilMap;
import com.malk.Util.UtilString;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.h3yun.CYClient;
import com.malk.shanghaishiwei.server.HTConf;
import com.malk.shanghaishiwei.service.HTClient;
import com.malk.shanghaishiwei.service.HTService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class HTImplService implements HTService {

    @Autowired
    private HTClient htClient;

    @Autowired
    private CYClient cyClient;

    @Autowired
    private DDClient ddClient;

    /**
     * 审批通过
     */
    @Override
    public String approveContract(Map<String, String> data) {

        Map bizObject = ((List<Map>) JSON.parse(data.get("bizObject"))).get(0);

        // prd: 非政采合同, 使用流水号
        String origionContractCode = UtilMap.getString(bizObject, "origionContractCode", UtilMap.getString(bizObject, "SeqNo"));
        bizObject.put("origionContractCode", origionContractCode);
        htClient.addPurchaseContact(bizObject); // 新增合同
        // 同步条款
        List<Map> termsObject = (List<Map>) JSON.parse(data.get("termsObject"));
        for (Map term : termsObject) {
            term.put("origionContractCode", origionContractCode);
            term.put("agreementType", HTConf.getAgreementType(term));
            htClient.addTermsContract(term);
        }
        return addAttachments(data, origionContractCode);
    }

    /**
     * 添加附件
     */
    @Override
    public String addAttachments(Map<String, String> data, String origionContractCode) {
        // prd: 非政采合同, 使用流水号
        if (StringUtils.isBlank(origionContractCode)) {
            Map bizObject = ((List<Map>) JSON.parse(data.get("bizObject"))).get(0);
            origionContractCode = UtilMap.getString(bizObject, "origionContractCode", UtilMap.getString(bizObject, "SeqNo"));
        }
        // 同步附件
        List<Map> attachments = (List<Map>) JSON.parse(data.get("attachments"));
        if (ObjectUtil.isNull(attachments)) {
            return "";
        }
        List<String> fileNames = new ArrayList<>();
        // prd: 合同接口文件名称不能重复, 添加创建时间标识, 兼容多个附件场景
        int index = 1;
        for (Map attachment : attachments) {
            File file = cyClient.saveFromTemporaryUrl(ddClient.getAccessToken(), attachment);
            String fileName = UtilMap.getString(attachment, "FileName");
            String fileType = fileName.substring(fileName.lastIndexOf(".")).toLowerCase();
            fileName = origionContractCode + "_" + UtilDateTime.format(new Date(), "yyyyMMddHHmmss") + "_" + index + "_" + fileName;
            Map body = UtilMap.map("origionContractCode, fileType, fileName", origionContractCode, fileType, fileName);
            body.put("contractType", 1); // 采购合同
            body.put("Scanning", UtilFile.fileConvertToByteArray(file));
            htClient.addAttachmentContract(body);
            fileNames.add(fileName); // 拼接与氚云记录, 服务删除逻辑
            index++;
        }
        // 兼容后补盖章附件
        String updateNames = String.join("; ", fileNames);
        String recordNames = data.get("fileNames");
        if (StringUtils.isNotBlank(recordNames)) {
            updateNames = recordNames + "; " + updateNames;
        }
        return updateNames;
    }

    /**
     * 批量删除
     */
    @Override
    public void deleteContract(Map<String, String> data) {

        List<Map> bizObjects = ((List<Map>) JSON.parse(data.get("bizObjects")));
        for (Map bizObject : bizObjects) {

            // 审批通过生效数据
            if (UtilMap.getInt(bizObject, "Status") != 1) {
                log.info("未审批通过数据, {}", bizObject);
                continue;
            }
            // prd: 非政采合同, 使用流水号
            String origionContractCode = UtilMap.getString(bizObject, "origionContractCode", UtilMap.getString(bizObject, "SeqNo"));
            // 删除附件
            if (UtilString.isNotBlankCompatNull(UtilMap.getString(bizObject, "fileNames"))) {
                String[] fileNames = UtilMap.getString(bizObject, "fileNames").split("; ");
                for (String fileName : fileNames) {
                    String fileType = fileName.substring(fileName.lastIndexOf(".")).toLowerCase();
                    htClient.deleteAttachmentContract(origionContractCode, fileType, fileName);
                }
            }
            // 删除条款
            List<Map> termsObject = (List<Map>) JSON.parse(data.get("termsObject"));
            for (Map term : termsObject) {
                htClient.deleteTermsContract(origionContractCode, UtilMap.getString(term, "dateTime"), HTConf.getAgreementType(term));
            }
            // 删除合同
            htClient.deletePurchaseContract(origionContractCode);
        }
    }
}
