package com.malk.xiding.service.impl;

import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilDateTime;
import com.malk.Util.UtilMap;
import com.malk.Util.UtilMc;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.xiding.service.GSClient;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
@Slf4j
public class GSImplClient implements GSClient {

    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDService ydService;

    /**
     * 月度工时汇总
     */
    @Synchronized
    @Override
    public void collectMonthReport() {

        // 每月3号: 上月第一天 ~ 最后一天
        long sStart = UtilDateTime.parseDateTime(UtilDateTime.firstDayOfNextMonth(new Date())).getTime();
        long sEnd = UtilDateTime.parseDateTime(UtilDateTime.lastDayOfNextMonth(new Date())).getTime();
        List<Map> dataList = ydService.queryAllFormData(YDParam.builder()
                .formUuid("FORM-L8966281J9RDFKMOEEUDP92ITHFF3YL1JCULL0")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_llucjaqc", Arrays.asList(sStart, sEnd))))
                .build(), false)
                .stream().map(item -> (Map) item.get("formData")).collect(Collectors.toList());
        // 按照人员拼装数据, 人员ID唯一性过滤
        List<Map> formList = dataList.stream().filter(UtilMc.distinctByKey(item -> item.get("textField_lm4nch8r"))).collect(Collectors.toList());
        for (Map tData : formList) {
            String tUserId = tData.get("textField_lm4nch8r").toString();
            // 数据汇总, 按照人员发布
            String ymText = UtilDateTime.format(new Date(sStart), "yyyy-MM");
            long total = ydClient.queryData(YDParam.builder()
                    .formUuid("FORM-TD966Z81I9ODTCY66GH345S03VW03JJF6EQLL5")
                    .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm5n9tjp, textField_lm5pchfd", ymText, tUserId)))
                    .build(), YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
            // prd 3号前若是未完全填报, 先删除月度汇总记录, 再进行手动同步
            if (total > 0) {
                continue;
            }
            Map formData = UtilMap.map("dateField_llqe7fg9, employeeField_llqe7fgd, textField_llugjn6g, textField_llugjn6i, textField_lm5n9tjp, textField_lm5pchfd", sStart, tData.get("employeeField_lm4n9bhf_id"), tData.get("textField_llomwm9a"), tData.get("textField_llomwm9b"), ymText, tUserId);
            // 月度确认: 明细数据汇总
            AtomicReference<Float> totalHour = new AtomicReference<>((float) 0);
            List<Map> details = dataList.stream().filter(item -> tUserId.equals(item.get("textField_lm4nch8r"))).
                    map(item -> {
                        totalHour.updateAndGet(v -> v + UtilMap.getFloat(item, "numberField_llucjaqe"));
                        return UtilMap.map("dateField_llvxfc4i, dateField_llvxfc4k, textField_llugjn6c, textField_llugjn6e, numberField_llqe7fgf, numberField_lm5pnmvi, textField_lm5rguwu, numberField_lm8i275h, numberField_llun9w37, numberField_lm8i275h, textField_lm7cfak9", "dateField_llqei2jn, dateField_llvostcl, textField_llucjaqg, textField_llucjaql, numberField_llucjaqe, numberField_llucjaqe, textField_lm5rguwu, numberField_lm8i275h, numberField_llun9w37, numberField_lm8i275h, textField_lm7cfak9", item);
                    })
                    .collect(Collectors.toList());
            // 月度总工时 & 日期排序
            formData.put("numberField_lm5n9tjo", totalHour.get());
            formData.put("numberField_lm5pnmvg", totalHour.get());
            Collections.sort(details, Comparator.comparingLong(o -> Long.parseLong(String.valueOf(o.get("dateField_llvxfc4i")))));
            formData.put("tableField_llqe7fgb", details);
            YDParam ydParam = YDParam.builder()
                    .formUuid("FORM-TD966Z81I9ODTCY66GH345S03VW03JJF6EQLL5")
                    .processCode("TPROC--TD966Z81I9ODTCY66GH345S03VW03KJF6EQLL6")
                    .formDataJson(JSON.toJSONString(formData))
                    .build();
            ydClient.operateData(ydParam, YDConf.FORM_OPERATION.start);
        }
    }

    /**
     * 月度异常上报
     */
    @Synchronized
    @Override
    public boolean exceptionMonthReport() {

        // 每月10号: 上月第一天 ~ 最后一天
        long sStart = UtilDateTime.parseDateTime(UtilDateTime.firstDayOfNextMonth(new Date())).getTime();
        long sEnd = UtilDateTime.parseDateTime(UtilDateTime.lastDayOfNextMonth(new Date())).getTime();
        // 薪资汇总
        List<Map> dataList_xz = ydService.queryAllFormData(YDParam.builder()
                .formUuid("FORM-3J866L81SSMDFKAGDVFJRDUK3TUW37K70MOLL9")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_lluli8zb", Arrays.asList(sStart, sEnd))))
                .build(), false)
                .stream().map(item -> (Map) item.get("formData")).collect(Collectors.toList());
        // 无固定ip, 轮询机制
        if (dataList_xz.size() == 0) {
            return false;
        }
        // 月度汇总
        List<Map> dataList_hz = ydService.queryAllFormData(YDParam.builder()
                .formUuid("FORM-TD966Z81I9ODTCY66GH345S03VW03JJF6EQLL5")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_llqe7fg9", Arrays.asList(sStart, sEnd))))
                .build(), true)
                .stream().map(item -> {
                    item.putAll((Map) item.get("data"));
                    return item; // 保留 formInstanceId
                }).collect(Collectors.toList());
        for (Map data : dataList_xz) {
            float attendanceHour = UtilMap.getFloat(data, "numberField_llpxvy5i");
            String userId = ((List<String>) data.get("employeeField_llpxvy5d_id")).get(0);
            Optional optional = dataList_hz.stream().filter(item -> userId.equals(item.get("textField_lm5pchfd"))).findAny();
            if (optional.isPresent()) {
                Map form = (Map) optional.get();
                // 未审批通过, 先过滤, 避免修改冲突 [拒绝已关闭]
                if (!"agree".equals(form.get("approvedResult"))) {
                    continue;
                }
                // 考勤工时不等于上报工时, 触发异常
                if (attendanceHour != UtilMap.getFloat(form, "numberField_lm5pnmvg")) {
                    long total = ydClient.queryData(YDParam.builder()
                            .formUuid("FORM-8Y866XB1G4NDRF8RCHWJRB5EQPI13TD3I8ULLT")
                            .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm5n9tjp, textField_lm5pchfd", "textField_lm5n9tjp, textField_lm5pchfd", form)))
                            .build(), YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
                    // 重复发起过滤
                    if (total > 0) {
                        continue;
                    }
                    Map formData = UtilMap.map("dateField_llu8im27, employeeField_llu8im2a, textField_llujr4wm, textField_llujr4wn, numberField_lm5qn2sf, textField_lm5n9tjp, textField_lm5pchfd, numberField_lm5ntjsu", "dateField_llqe7fg9, employeeField_llqe7fgd_id, textField_llugjn6g, textField_llugjn6i, numberField_lm5qn2sf, textField_lm5n9tjp, textField_lm5pchfd, numberField_lm5pnmvg", form);
                    formData.put("numberField_llu8im2e", attendanceHour);
                    log.info("异常记录, {}", JSON.toJSONString(formData));
                    List<Map> details = (List<Map>) form.get("tableField_llqe7fgb");
                    if (details.size() == 50) {
                        details = ydService.queryDetails(YDParam.builder()
                                .formUuid("FORM-TD966Z81I9ODTCY66GH345S03VW03JJF6EQLL5")
                                .formInstanceId(form.get("processInstanceId").toString())
                                .tableFieldId("tableField_llqe7fgb")
                                .build());
                    }
                    // 取值修改后工时
                    formData.put("tableField_llu8im28", details.stream().map(item ->
                            UtilMap.map("dateField_llvz24wf, dateField_llvz24wh, textField_llujr4wj, textField_llujr4wk, numberField_llu8im2c, numberField_lm5ntjsw, textField_lm5rguwu, textField_lm7cfak9, numberField_llun9w37, numberField_lm8i275h", "dateField_llvxfc4i, dateField_llvxfc4k, textField_llugjn6c, textField_llugjn6e, numberField_lm5pnmvi, numberField_lm5pnmvi, textField_lm5rguwu, textField_lm7cfak9, numberField_llun9w37, numberField_lm8i275h", item))
                            .collect(Collectors.toList()));
                    YDParam ydParam = YDParam.builder()
                            .formUuid("FORM-8Y866XB1G4NDRF8RCHWJRB5EQPI13TD3I8ULLT")
                            .processCode("TPROC--8Y866XB1G4NDRF8RCHWJRB5EQPI13UD3I8ULLU")
                            .formDataJson(JSON.toJSONString(formData))
                            .build();
                    ydClient.operateData(ydParam, YDConf.FORM_OPERATION.start);
                }
            } else {
                String ymDate = UtilDateTime.format(new Date(UtilMap.getLong(data, "dateField_lluli8zb")), "yyyy-MM");
                String tUserId = ((List<String>) data.get("employeeField_llpxvy5d_id")).get(0);
                long total = ydClient.queryData(YDParam.builder()
                        .formUuid("FORM-8Y866XB1G4NDRF8RCHWJRB5EQPI13TD3I8ULLT")
                        .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm5n9tjp, textField_lm5pchfd", ymDate, userId)))
                        .build(), YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
                // 重复发起过滤
                if (total > 0) {
                    continue;
                }
                // 若未填报工时 - 直接发起异常
                Map formData = UtilMap.map("dateField_llu8im27, employeeField_llu8im2a, textField_llujr4wm, textField_llujr4wn, numberField_llu8im2e", "dateField_lluli8zb, employeeField_llpxvy5d_id, textField_llpxvy5k, textField_llpxvy5l, numberField_llpxvy5i", data);
                formData.putAll(UtilMap.map("textField_lm5n9tjp, textField_lm5pchfd, numberField_lm5ntjsu", ymDate, tUserId, ""));
                YDParam ydParam = YDParam.builder()
                        .formUuid("FORM-8Y866XB1G4NDRF8RCHWJRB5EQPI13TD3I8ULLT")
                        .processCode("TPROC--8Y866XB1G4NDRF8RCHWJRB5EQPI13UD3I8ULLU")
                        .formDataJson(JSON.toJSONString(formData))
                        .build();
                ydClient.operateData(ydParam, YDConf.FORM_OPERATION.start);
            }
        }
        return true;
    }

    /**
     * 工时提交校验
     */
    @Override
    public void validateManHour(String userId, Map<String, String> details) {

        // 工时填报
        for (String date : details.keySet()) {
            List<Map> dataList = ydService.queryAllFormData(YDParam.builder()
                    .formUuid("FORM-L8966281J9RDFKMOEEUDP92ITHFF3YL1JCULL0")
                    .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm4nch8r, textField_lm7cfak9", userId, date)))
                    .build(), false)
                    .stream().map(item -> (Map) item.get("formData")).collect(Collectors.toList());
            if (dataList.size() == 0) {
                continue;
            }
            float hour = UtilMap.getFloat(details, date);
            for (Map data : dataList) {
                hour += UtilMap.getFloat(data, "numberField_llucjaqe");
                if (hour > 8.0f) {
                    McException.exceptionAccess(date + " 累计填报工时已超过8h!");
                }
            }
        }
    }

    /**
     * 消息通知 [异常提醒: 当月处理完成; 填报提醒: 当月1号提醒]
     */
    @Override
    public void noticeTemplate(boolean isAdmin) {

        // 推送人员
        Map conditions = isAdmin ? UtilMap.map("radioField_lmynhto8", "是") : new HashedMap();
        List<Map> dataList = ydService.queryAllFormData(YDParam.builder()
                .formUuid("FORM-IH966T61S6ND3ZHX6WEK2DFHHWLD3W44DMOLL0")
                .searchFieldJson(JSON.toJSONString(conditions))
                .build(), false);
        // 推送内容
        long timestamp = new Date().getTime();
        String type = isAdmin ? "异常完成提醒" : "工时填报提醒";
        String title = "【系统通知】" + type;
        String content = isAdmin ? "全部异常工时修改已审批完成，谢谢！" : "请您及时填写本月的工时，谢谢！";

        List<Map> formList = dataList.stream().map(item -> {
            Map form = (Map) item.get("formData");
            return UtilMap.map("employeeField_lmymvs91, dateField_lmymvs93, radioField_lmymvs9j, textField_lmymvs95, textareaField_lmymvs99",
                    form.get("employeeField_llpzbxhc_id"), timestamp, type, title, content);
        }).collect(Collectors.toList());

        // 批量推送
        formList.forEach(data -> {
            ydClient.operateData(YDParam.builder()
                    .formUuid("FORM-TP866D914SEE7WPKF5LHS7P0LXXZ1QE3TMYMLJ")
                    .formDataJson(JSON.toJSONString(data))
                    .build(), YDConf.FORM_OPERATION.create);
        });

    }
}
