package com.malk.xiding.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilDateTime;
import com.malk.Util.UtilMap;
import com.malk.Util.UtilMc;
import com.malk.server.common.McException;
import com.malk.server.fxiaoke.FXKConf;
import com.malk.service.dingtalk.*;
import com.malk.service.fxiaoke.FXKClient;
import com.malk.xiding.repository.dao.XdDdApproveRecordDao;
import com.malk.xiding.repository.dao.XdDdApproveRecordDao2;
import com.malk.xiding.repository.dao.XdDdFxkRelationDao;
import com.malk.xiding.repository.dao.XdFxkDdAttendanceDao;
import com.malk.xiding.repository.entity.XdDdApproveRecordPo;
import com.malk.xiding.repository.entity.XdDdFxkRelationPo;
import com.malk.xiding.repository.entity.XdFxkDdAttendancePo;
import com.malk.xiding.server.XDConf;
import com.malk.xiding.service.XDService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class XDImplService implements XDService {

    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    @Autowired
    private DDClient_Contacts ddClient_contacts;

    @Autowired
    private DDClient_Attendance ddClient_attendance;

    @Autowired
    private DDService ddService;

    @Autowired
    private FXKClient fxkClient;

    @Autowired
    private FXKConf fxkConf;

    @Autowired
    private XdDdApproveRecordDao ddApproveRecordDao;

    @Autowired
    private XdDdFxkRelationDao ddFxkRelationDao;

    /**
     * 同步审批单
     * -
     * 审批通过: 外出 + 出差. 后补审批自动同步当月
     */
    @Override
    public void syncApprove(long beginTime, long finishTime, String tmpCode, String type) {

        log.info("同步{}审批单: {} ~ {}", type, new Date(beginTime), new Date(finishTime));
        // 同步审批通过审批单 & 人员信息 [差集]
        Map extInfo = UtilMap.map("statuses", Arrays.asList("COMPLETED"));
        List<String> dataList = ddClient_workflow.getInstanceIds_all(ddClient.getAccessToken(), tmpCode, beginTime, finishTime, extInfo);
        List<String> poList = ddApproveRecordDao.findAllByInstanceIdIn(dataList).stream().map(item -> item.getInstanceId()).collect(Collectors.toList());
        List<String> syncList = dataList.stream().filter(item -> !poList.contains(item)).collect(Collectors.toList());
        log.info("[{}] 实例列表, dataList = {}, poList = {}, syncList = {}", type, dataList.size(), poList.size(), syncList.size());
        syncList.forEach(instanceId -> {
            // prd 1.23 避免限流, 已调整定时任务错峰执行
            try {
                Thread.sleep(400);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            Map record = ddClient_workflow.getProcessInstanceId(ddClient.getAccessToken(), instanceId);
            if (!"agree".equals(record.get("result"))) {
                return;
            }
            String userId = String.valueOf(record.get("originatorUserId"));
            XdDdFxkRelationPo userPo = _syncUserInfo(userId); // 同步人员对照表
            // 离职人员不同步 & 未匹配人员暂时不同步, 手机号匹配可重新拉取
            if (ObjectUtil.isNull(userPo)) {
                return;
            }
            // 写入审批单记录用于考勤同步
            String startTime = null, endTime = null;
            List<Map> formComponentValues = (List<Map>) record.get("formComponentValues");
            if (tmpCode.equals(XDConf.TMP_CODE_CC)) {
                String schedule = (String) formComponentValues.stream().filter(item -> "itinerary".equals(item.get("bizAlias"))).findAny().get().get("value");
                // ppExt: 出差申请单为明细组件, 存在多条情况 23.07.03
                List<Map> itineraryList = ((List<Map>) JSON.parse(schedule));
                for (Map itinerary : itineraryList) {
                    List<Map> rowValue = (List<Map>) itinerary.get("rowValue");
                    startTime = (String) rowValue.stream().filter(item -> "startTime".equals(item.get("bizAlias"))).findAny().get().get("value");
                    endTime = (String) rowValue.stream().filter(item -> "endTime".equals(item.get("bizAlias"))).findAny().get().get("value");
                    _approveRecord(instanceId, type, userPo, userId, startTime, endTime);
                }
            } else {
                formComponentValues = (List<Map>) record.get("formComponentValues");
                startTime = (String) formComponentValues.stream().filter(item -> "startTime".equals(item.get("bizAlias"))).findAny().get().get("value");
                endTime = (String) formComponentValues.stream().filter(item -> "finishTime".equals(item.get("bizAlias"))).findAny().get().get("value");
                _approveRecord(instanceId, type, userPo, userId, startTime, endTime);
            }
        });
    }

    // ppExt: 出差申请单为明细组件, 存在多条情况 23.07.03
    void _approveRecord(String instanceId, String type, XdDdFxkRelationPo userPo, String userId, String startTime, String endTime) {
        // 不区分半天考勤时段, 全天都可进行打卡
        Date sTime = UtilDateTime.parseDateTime(startTime.split(" ")[0] + " 00:00:00");
        Date eTime = UtilDateTime.parseDateTime(endTime.split(" ")[0] + " 23:59:59");
        XdDdApproveRecordPo approveRecordPo = XdDdApproveRecordPo.builder()
                .userId(userId)
                .userName(userPo.getName())
                .openUserId(userPo.getIdFxxk())
                .instanceId(instanceId)
                .startTime(sTime)
                .endTime(eTime)
                .type(type)
                .build();
        // prd 2023-07 出差考勤同步仅适用于营销中心以及其子部门
        if (type.equals("出差")) {
            // 离职人员同步过滤
            try {
                approveRecordPo.setMarketing(ddService.matchDepartment(ddClient.getAccessToken(), userId, Arrays.asList(37263540L)));
            } catch (McException e) {
                log.error(e.getMessage(), e);  // 记录错误日志
            }
        }
        ddApproveRecordDao.save(approveRecordPo);
        log.info("同步出差和人员信息, {}, {}", userPo, approveRecordPo);
        _attendanceSync(sTime, eTime, userPo.getIdFxxk(), userId, type, approveRecordPo.isMarketing());  // 兼容后补审批记录
    }

    // 同步纷享销客人员信息
    private XdDdFxkRelationPo _syncUserInfo(String userId) {
        // 更新人员对照记录表, 包括失败备注
        XdDdFxkRelationPo userPo = ddFxkRelationDao.findByIdDd(userId);
        if (ObjectUtil.isNull(userPo) || StringUtils.isNotBlank(userPo.getRemark())) {
            Map userInfo_dd = null;
            try {
                userInfo_dd = ddClient_contacts.getUserInfoById(ddClient.getAccessToken(), userId);
            } catch (McException e) {
                log.error(e.getMessage(), e);  // 记录错误日志
                return null;  // 离职人员不同步: 历史数据处理
            }
            XdDdFxkRelationPo relationPo = XdDdFxkRelationPo.builder()
                    .idDd(userId)
                    .name(String.valueOf(userInfo_dd.get("name")))
                    .mobile(String.valueOf(userInfo_dd.get("mobile")))
                    .build();
            // 若是同步异常记录, 执行更新
            if (ObjectUtil.isNotNull(userPo)) {
                relationPo.setId(userPo.id);
                relationPo.setCreateTime(userPo.getCreateTime());
            }
            Map userInfo_fxk;
            try {
                userInfo_fxk = fxkClient.getUserByMobile(fxkClient.getAccessToken(), fxkConf.getCorpId(), String.valueOf(userInfo_dd.get("mobile"))).get(0);
                relationPo.setIdFxxk(String.valueOf(userInfo_fxk.get("openUserId")));
            } catch (McException e) {
                relationPo.setRemark(e.getMessage());
                log.error(e.getMessage(), e);  // 记录错误日志
            }
            userPo = relationPo;  // 同步信息, 储存
            ddFxkRelationDao.save(relationPo);
        }
        // 未匹配纷享销客不存储, 信息更正后程序还可重新拉取
        if (StringUtils.isBlank(userPo.getIdFxxk())) {
            return null;
        }
        return userPo;
    }

    @Autowired
    private XdDdApproveRecordDao2 approveRecordDao2;

    @Autowired
    private XdFxkDdAttendanceDao attendanceDao;

    /**
     * 同步考勤记录
     * -
     * 11点前，同步到钉钉9点
     * 16点后，同步到钉钉18点
     * 11-16点之间，同步真实打卡时间，但不影响考勤结果
     * 0-5点的打卡记为前一天18点
     */
    @Override
    public void syncAttendance(int page, Date sTime, Date eTime) {
        log.info("考勤同步第{}批: {} ~ {}", page, sTime, eTime);
        Page<XdDdApproveRecordPo> poPage = approveRecordDao2.findAllByEndTimeAfter(sTime, PageRequest.of(page, 40));
        // 全量同步
//        Page<XdDdApproveRecordPo> poPage = approveRecordDao2.findAll(PageRequest.of(page, 40));
//        Page<XdDdApproveRecordPo> poPage = approveRecordDao2.findAllByOpenUserId("FSUID_4ED8868EE972F63FBBB19318EEB36455", PageRequest.of(page, 40));
        if (poPage.isEmpty()) {
            return;
        }
        // id去重 [查询单个用户可考虑去重]
//        List<XdDdApproveRecordPo> poList = poPage.getContent();
        List<XdDdApproveRecordPo> poList = poPage.getContent().stream().filter(UtilMc.distinctByKey(XdDdApproveRecordPo::getOpenUserId)).collect(Collectors.toList());
        List<String> openUserIds = poList.stream().map(item -> item.getOpenUserId()).collect(Collectors.toList());
        List<Map> attendanceList = fxkClient.getAttendance_All(fxkClient.getAccessToken(), fxkConf.getCorpId(), sTime.getTime(), eTime.getTime(), openUserIds);

        log.info("同步人员数量 = {}, 考勤数量 = {}", openUserIds.size(), attendanceList.size());
        attendanceList.forEach(item -> {
            XdFxkDdAttendancePo attendancePo = JSON.parseObject(JSON.toJSONString(item), XdFxkDdAttendancePo.class);
            attendancePo.setIdDd(poList.stream().filter(row -> row.getOpenUserId().equals(attendancePo.getOpenUserId())).findAny().get().getUserId());
            _attendanceRule(attendancePo);
        });
        if (!poPage.isLast()) {
            syncAttendance(poPage.getNumber() + 1, sTime, eTime);
        }
    }

    /// 后补审批单, 同步考勤
    private void _attendanceSync(Date sTime, Date eTime, String openUserId, String ddUserId, String type, boolean marketing) {
        // [人员匹配 & 开始时间小于当前同步时间, 若当前时间大于结束时间取结束时间]
        Date syncTime = eTime.after(new Date()) ? new Date() : eTime;
        if (sTime.after(syncTime) || StringUtils.isBlank(openUserId)) {
            return;
        }
        if (type.equals("外出")) {
            // 不跨月统计, 纷享销客查询范围仅支持40天内
            if (sTime.getMonth() != eTime.getMonth()) {
                sTime = new Date(sTime.getYear(), eTime.getMonth(), 1);
            }
            List<Map> attendanceList = fxkClient.getAttendance_All(fxkClient.getAccessToken(), fxkConf.getCorpId(), sTime.getTime(), eTime.getTime(), Arrays.asList(openUserId));
            log.info("后补人员 = {}, 考勤数量 = {}", openUserId, attendanceList.size());
            attendanceList.forEach(item -> {
                XdFxkDdAttendancePo attendancePo = JSON.parseObject(JSON.toJSONString(item), XdFxkDdAttendancePo.class);
                attendancePo.setIdDd(ddUserId);
                _attendanceRule(attendancePo);
            });
        } else {
            // 仅同步营销部门人员
            if (marketing) {
                // 提交出差后, 就会有对应的排班, 无需重复查询; 同步时匹配出差结束时间, 不写入未来时间考勤
                LocalDateTime ssTime = UtilDateTime.convertToLocalDateTimeFromDate(sTime);
                while (ssTime.isBefore(UtilDateTime.convertToLocalDateTimeFromDate(syncTime))) {
                    log.info("出差后补 = {}, 考勤时间 = {}", ddUserId, ssTime);
                    _attendanceRule(ddUserId, ddFxkRelationDao.findByIdDd(ddUserId).getName(), UtilDateTime.formatLocalDate(ssTime.toLocalDate()) + " 09:00:00");
                    ssTime = ssTime.plusDays(1);
                }
            }
        }
    }

    // 考勤写入规则
    private void _attendanceRule(XdFxkDdAttendancePo attendancePo) {
        // 11 - 16 之间之间同步打卡记录
        long check = attendancePo.getCheckTime().getTime();
        int hour = attendancePo.getCheckTime().getHours();
        if (hour < 11) {
            // 凌晨5点前打卡记为第前天下班卡
            if (hour < 5) {
                check = UtilDateTime.parseDateTime(UtilDateTime.formatDate(new Date(attendancePo.getCheckTime().getTime() - (hour + 1) * 60 * 60 * 1000)) + " 18:00:00").getTime();
            } else {
                check = UtilDateTime.parseDateTime(UtilDateTime.formatDate(attendancePo.getCheckTime()) + " 09:00:00").getTime();
            }
        }
        if (hour >= 16) {
            check = UtilDateTime.parseDateTime(UtilDateTime.formatDate(attendancePo.getCheckTime()) + " 18:00:00").getTime();
        }
        try {
            boolean hasRecord = attendanceDao.existsByIdDdAndCheckTimeEquals(attendancePo.getIdDd(), attendancePo.getCheckTime());
            if (hasRecord) {
                log.info("存在考勤, {}, {}", attendancePo.getUserName(), attendancePo.getCheckTime());
                return;
            }
            // 中间时段, 匹配是否最晚打卡, 影响钉钉考勤结果
            if (check == attendancePo.getCheckTime().getTime()) {
                String sTime = UtilDateTime.formatDateTime(attendancePo.getCheckTime());
                String eTime = UtilDateTime.formatLocalDate(UtilDateTime.convertToLocalDateTimeFromDate(attendancePo.getCheckTime()).plusDays(1).toLocalDate()) + " 05:00:00";
                List records = ddClient_attendance.listAttendanceRecord(ddClient.getAccessToken(), Arrays.asList(attendancePo.getIdDd()), sTime, eTime);
                if (records.size() > 0) {
                    log.info("避免覆盖, {}, {}, {}, {}", attendancePo.getUserName(), sTime, eTime, records.size());
                    return;
                }
            }
            String ss = "纷享销客同步" + ": " + UtilDateTime.formatDateTime(new Date(attendancePo.getCheckTime().getTime()));
            // 燕江 0953580166764920
            ddClient_attendance.uploadAttendanceRecord(ddClient.getAccessToken(), attendancePo.getIdDd(), ss, "纷享销客同步", null, check);
            attendancePo.setRemark("同步成功");
            attendanceDao.save(attendancePo);
            log.info("写入考勤, {}, {}", attendancePo.getUserName(), attendancePo.getCheckTime());
        } catch (McException e) {
            log.error(e.getMessage(), e);  // 记录错误日志
        }
    }

    /**
     * 同步出差记录考勤 [营销中心]
     * -
     * 18点同步, 自动上传 [同步时匹配出差时间 [仅同步: 出差开始时间/同步开始 ~ 同步结束/出差结束时间]
     * 废弃同步排班原因: 提交出差后, 就会有对应的排班, 无需重复查询. 且不同出差单起止时间不一致, 需要处理
     */
    @Override
    public void syncAttendanceForCC(int page, Date sTime, Date eTime) {
        Page<XdDdApproveRecordPo> poPage = approveRecordDao2.findAllByEndTimeAfterAndTypeAndMarketing(sTime, "出差", true, PageRequest.of(page, 50));
        // 不能去重, 批量同步时, 可能一个时间段内会有多个出差单
        List<XdDdApproveRecordPo> poList = poPage.getContent();
        poList.forEach(po -> {
            // 提交出差后, 就会有对应的排班, 无需重复查询; 同步时匹配出差时间 [仅同步: 出差开始时间/同步开始 ~ 同步结束/出差结束时间]
            Date ssTime = sTime.before(po.getStartTime()) ? po.getStartTime() : sTime;
            Date eeTime = eTime.after(po.getEndTime()) ? po.getEndTime() : eTime;
            if (ssTime.after(eeTime)) {
                return;
            }
            log.info("同步出差记录考勤, {}, {} ~ {}", ddFxkRelationDao.findByIdDd(po.getUserId()).getName(), ssTime, eeTime);
            for (int day = ssTime.getDate(); day <= eeTime.getDate(); day++) {
                _attendanceRule(po.getUserId(), ddFxkRelationDao.findByIdDd(po.getUserId()).getName(), UtilDateTime.formatDate(new Date(ssTime.getYear(), eeTime.getMonth(), day)) + " 09:00:00");
            }
        });
        if (!poPage.isLast()) {
            syncAttendanceForCC(poPage.getNumber() + 1, sTime, eTime);
        }
    }

    // 考勤写入规则 [23.07 出差自动同步]
    private void _attendanceRule(String userId, String userName, String date) {
        XdFxkDdAttendancePo attendancePo = XdFxkDdAttendancePo.builder()
                .idDd(userId)
                .checkTime(UtilDateTime.parseDateTime(date))
                .userName(userName)
                .build();
        try {
            boolean hasRecord = attendanceDao.existsByIdDdAndCheckTimeEquals(attendancePo.getIdDd(), attendancePo.getCheckTime());
            if (hasRecord) {
                log.info("存在考勤, {}, {}", attendancePo.getUserName(), attendancePo.getCheckTime());
                return;
            }
            String ss = "出差考勤同步" + ": " + UtilDateTime.formatDateTime(attendancePo.getCheckTime());
            // 燕江 0953580166764920
            ddClient_attendance.uploadAttendanceRecord(ddClient.getAccessToken(), attendancePo.getIdDd(), ss, "出差考勤同步", null, attendancePo.getCheckTime().getTime());
            attendancePo.setRemark("同步成功");
            attendanceDao.save(attendancePo);
            log.info("写入考勤, {}, {}", attendancePo.getUserName(), attendancePo.getCheckTime());
        } catch (McException e) {
            log.error(e.getMessage(), e);  // 记录错误日志
        }
        // 同时写入下班考勤
        if (date.endsWith(" 09:00:00")) {
            _attendanceRule(userId, userName, date.replace(" 09:00:00", " 18:00:00"));
        }
    }
}
