package com.malk.server.dingtalk;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilHttp;
import com.malk.server.common.McException;
import com.malk.server.common.VenR;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 返回值配置参考McR
 */
@Data
@NoArgsConstructor
public class DDR_New<T> extends VenR {

    private String code;

    private String message;

    private Object data;

    private boolean success;

    private T result;

    /**
     * 审批实例id
     */
    private String instanceId;

    /**
     * 创建时间
     */
    private Date createTimeGMT;

    /**
     * 修改时间
     */
    private Date modifiedTimeGMT;

    /**
     * 流程标题
     */
    private String title;

    /**
     * 流程实例ID
     */
    private String processInstanceId;

    /**
     * 流程实例当前任务执行人列表
     */
    private List actionExecutor;

    /**
     * 流程编码
     */
    private String formUuid;

    /**
     * 流程编码
     */
    private String processCode;

    /**
     * 实例状态，取值：RUNNING：运行中, TERMINATED：已终止, COMPLETED：已完成, ERROR：异常
     */
    private String instanceStatus;

    /**
     * 审批结果，取值：agree：同意; disagree：拒绝
     */
    private String approvedResult;

    ////  宜搭数据  ////

    /**
     * 列表查询数据
     */
    private long pageNumber;

    private long totalCount;

    /**
     * 实例ID详情
     */
    private Map originator;
    
    private String formInstId;

    private Map formData;

    ////  储存空间  ////

    /**
     * 上传唯一标识
     */
    private String uploadKey;

    /**
     * 文件存储类型。DINGTALK：钉钉统一存储驱动, ALIDOC：钉钉文档存储驱动, UNKNOWN：未知驱动
     */
    private String storageDriver;

    /**
     * 上传协议。HEADER_SIGNATURE：Header加签
     */
    private String protocol;

    /**
     * Header加签上传信息。说明: 当protocol参数传HEADER_SIGNATURE时，返回该字段
     */
    private Map headerSignatureInfo;

    /**
     * 文件信息
     */
    private Map dentry;

    ////  请求操作  ////

    /**
     * 格式化返回值
     */
    public static DDR_New formatResponse(String response) {
        DDR_New r = JSON.parseObject(response, DDR_New.class);
        r.assertSuccess();
        // 新版本: 若存在code则失败, 否则直接返回请求字段, 无状态包装
        return JSON.parseObject(response, DDR_New.class);
    }

    /**
     * 断言错误信息
     */
    @Override
    public void assertSuccess() {
        McException.assertException(ObjectUtil.isNotNull(code), code, message, "dingtalk_new");
    }

    /**
     * 通用post请求
     */
    public static DDR_New doPost(String url, Map header, Map param, Map body) {
        String rsp = UtilHttp.doPost(url, header, param, body);
        return formatResponse(rsp);
    }

    /**
     * 通用get请求
     */
    public static DDR_New doGet(String url, Map header, Map param) {
        String rsp = UtilHttp.doGet(url, header, param);
        return formatResponse(rsp);
    }
}
