package com.malk.service.aliwork.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.*;

@Slf4j
@Service
public class YDServiceImpl implements YDService {

    @Autowired
    private YDClient ydClient;

    /**
     * 操作数据 [异步] - 审批通过立即更新, 会无效
     */
    @Async
    @SneakyThrows
    @Override
    public Object operateData(YDParam ydParam, YDConf.FORM_OPERATION type) {
        Thread.sleep(3000);
        return ydClient.operateData(ydParam, type);
    }


    /**
     * 子表全部数据获取, 最大分页为50 [可考虑中间表思路]
     */
    @Override
    public List<Map> queryDetails(YDParam ydParam) {
        return _queryDetails(ydParam, new ArrayList<>());
    }

    // 递归查询
    private List<Map> _queryDetails(YDParam ydParam, List<Map> details) {
        ydParam.setPageSize(50);
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_details);
        details.addAll((List<Map>) ddr_new.getData());
        if (ddr_new.getTotalCount() != details.size()) {
            ydParam.setPageNumber(ydParam.getPageNumber() + 1);
            _queryDetails(ydParam, details);
        }
        return details;
    }

    /**
     * 查询全部 [主表]
     */
    @Override
    public List<Map> queryAllFormData(YDParam ydParam, boolean isProcess) {
        float pageSize = YDConf.PAGE_SIZE_LIMIT;
        // 查询数据量
        ydParam.setPageSize(1);
        long totalCount = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
        // 轮询累计数据
        List<Map> dataList = new ArrayList<>();
        ydParam.setCurrentPage(1);
        ydParam.setPageSize((int) pageSize);
        for (int page = 1; page <= Math.ceil(totalCount / pageSize); page++) {
            ydParam.setCurrentPage(page);
            dataList.addAll((List<Map>) ydClient.queryData(ydParam, isProcess ? YDConf.FORM_QUERY.retrieve_search_process : YDConf.FORM_QUERY.retrieve_search_form).getData());
        }
        return dataList;
    }

    /**
     * 全表复制
     */
    @Override
    public Object copyFormData(YDParam cParam, YDParam sParam) {

        Map<String, Object> formData = ydClient.queryData(cParam, YDConf.FORM_QUERY.retrieve_id).getFormData();

        Map newFormData = new HashMap();
        for (String compId : formData.keySet()) {
            if (compId.endsWith("_id") || compId.endsWith("_value")) {
                continue;
            }
            newFormData.put(compId, _formatCompData(compId, formData));
        }

        newFormData.put("employeeField_lia74rki", Arrays.asList(YDConf.PUB_ACCOUNT));

        sParam.setFormDataJson(JSON.toJSONString(newFormData));
        ydClient.operateData(sParam, YDConf.FORM_OPERATION.start);

        return null;
    }

    /// 格式化组件
    Object _formatCompData(String compId, Map<String, Object> formData) {

        // todo: 成员, 附件, 关联表单 ignoreDetails 忽略明细
//        List<Map> association = JSON.parseArray(String.valueOf(JSON.parse(String.valueOf(formData.get("associationFormField_lg0gmbi8_id")))), Map.class);
//        String formInstId = String.valueOf(association.get(0).get("instanceId"));

        if (compId.startsWith("tableField_")) {
            List<Map> details = (List<Map>) formData.get(compId);
            for (Map<String, Object> detail : details) {
                for (String compId_sub : detail.keySet()) {
                    detail.put(compId_sub, _formatCompData(compId_sub, detail));
                }
            }
        }
        if (compId.startsWith("employeeField_")) {
            return formData.get(compId + "_id");
        }
        if (compId.startsWith("associationFormField_")) {
            return JSON.parse(String.valueOf(formData.get(compId + "_id")));
        }

        return formData.get(compId);
    }

    /**
     * upsert方法
     */
    @Override
    public Object upsertFormData(String formUuid, Map condition, Map formData, UpsertLambda lambda) {
        YDParam ydParam = YDParam.builder()
                .formUuid(formUuid)
                .searchFieldJson(JSON.toJSONString(condition))
                .build();
        // 查询数据, 是否存在
        List<Map> dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();
        McException.assertAccessException(dataList.size() > 1, "upsert方法, 查询条件返回数据不唯一");
        // 异步回调, 查询结果
        if (ObjectUtil.isNotNull(lambda)) {
            Map form = lambda.dataList(dataList);
            // 若返回为空, 不执行
            if (ObjectUtil.isNull(form)) {
                return null;
            }
            // 回调数据, 写入表单
            formData.putAll(form);
        }
        if (dataList.size() > 0) {
            ydParam.setUpdateFormDataJson(JSON.toJSONString(formData));
            ydParam.setFormInstanceId(dataList.get(0).get("formInstanceId").toString());
            return ydClient.operateData(ydParam, YDConf.FORM_OPERATION.update);
        }
        formData.putAll(condition); // 新增写入查询条件数据
        ydParam.setFormDataJson(JSON.toJSONString(formData));
        return ydClient.operateData(ydParam, YDConf.FORM_OPERATION.create);
    }
}
